DAT programName  byte "PadSimulator110824e", 0
CON{{  
  Program to send simulated keypad
  data to "Black Box" which with
  combine serial lines to a single
  line and add a prefix to the data
  to indicate which keypad the data
  originated.
  The variable "delay" is used to
  change the pause between data
  transmissions.
  The array "sequence" gives to order
  of keypads to simulate.
  110824a remove control characters from
  data.
}}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _EndOfMessagePad = "#"
  _EndOfMessageBlackBox = 13    ' Used by Black Box for received messages only.
                                ' Black Box will use _EndOfMessagePad for all messages
                                ' transmitted from the Black Box.
                                
  _BlackBoxMessagePrefix = "?"  ' So receiving unit know message isn't from any keypad.
  
  '' Pin asignments
  '' Each "PadXTxPin" line is pulled-up to 3.3V with a 10K ohm resistor.
  _PadATxPin = 0 
  _PadBTxPin = 1
  _PadCTxPin = 2
  _PadDTxPin = 3                                     
  _PadETxPin = 4 
  _PadFTxPin = 5
  _PadGTxPin = 6  
  _OutputRxPin = 31
  _OutputTxPin = 30  
  _LedPin = 23                  ' Used as a "heartbeat" to indicate the Propeller
                                ' is active.                                          
    
  _PadBaud = 9600
  '_LargeBufferSize = 256        ' This should be at least as large as the largest message
                                ' the Black Box is expected to receive.
                                ' I made it large since there is still plenty of memory.
                                
  _SerialPortGroupSize = 4      ' Serial port object is limited to four ports
  _SerialPortGroups = 2         ' Using to of the four port serial objects
  
  _KeyPressBufferSize = 20
  _NumberOfOutputChoices = 10
  _NumberOfDelayChoices = 4
  ' Com enumeration
  #0, _PadA, _PadB, _PadC, _PadD, {second group} _PadE, _PadF, _PadG, _Output {0 - 7}
   
VAR

  long baudRate, oldBaudRate

  
OBJ

  Com0 :  "pcFullDuplexSerial4FC128"
  Com1 :  "pcFullDuplexSerial4FC128_2"
  led : "Flash"
  
PUB Main | localSize, localIndex, delay[_NumberOfDelayChoices], delayIndex,  {
         } activePad, outputIndex, characterIndex, doneFlag[_Output + 1]

  delay[0] := clkfreq
  delay[1] := clkfreq / 20
  delay[2] := clkfreq * 2
  delay[3] := clkfreq / 2
  
  waitcnt(clkfreq * 2 + cnt) 

  Led.Start(_LedPin, 150, 0)    ' hearbeat LED to show Propeller is working
    
  Com0.Init      
  Com0.AddPort(_PadA, -1, _PadATxPin, -1, -1, 0, 0, _PadBaud)
  Com0.AddPort(_PadB, -1, _PadBTxPin, -1, -1, 0, 0, _PadBaud)
  Com0.AddPort(_PadC, -1, _PadCTxPin, -1, -1, 0, 0, _PadBaud)
  Com0.AddPort(_PadD, -1, _PadDTxPin, -1, -1, 0, 0, _PadBaud)
  Com0.Start
                                                             
  Com1.Init      
  Com1.AddPort(_PadE - _SerialPortGroupSize, -1, _PadETxPin, -1, -1, 0, 0, _PadBaud)
  Com1.AddPort(_PadF - _SerialPortGroupSize, -1, _PadFTxPin, -1, -1, 0, 0, _PadBaud)
  Com1.AddPort(_PadG - _SerialPortGroupSize, -1, _PadGTxPin, -1, -1, 0, 0, _PadBaud)
  Com1.AddPort(_Output - _SerialPortGroupSize, _OutPutRxPin, _OutPutTxPin, -1, -1, 0, 0, _PadBaud)
  Com1.Start                                                           
 
  waitcnt(clkfreq/10 + cnt)

  delayIndex := 0
  outputIndex := 0
  Tbug(string(13, "Simple Test"))
  Tbug(string(13, "Four data sets from four pads."))
  waitcnt(clkfreq * 1 + cnt)
       
  repeat outputIndex from 0 to 3
    Tbug(string(13, "Output from pad"))
    TbugTx(prefixList[outputIndex])
    Tbug(string(" = "))
    localIndex := @padFakeOutput01 + (outputIndex * _KeyPressBufferSize)
    repeat
      TbugTx(byte[localIndex])' Send data to terminal window  
      Tx(outputIndex, byte[localIndex]) ' Send data to black box
    while byte[localIndex++] <> _EndOfMessagePad '"#"
    waitcnt(clkfreq * 1 + cnt) 

  Tbug(string(13, "End of ", 34, "Four data sets from four pads", 34, " Test.", 13))
  Tbug(string(13, "New test.  Interleave input from "))
  TbugDec(_Output)  
  Tbug(string(" pads."))
  Tbug(string(13, "All "))
  TbugDec(_Output)  
  Tbug(string(" serial lines will be used at once."))
  Tbug(string(13, "The Black Box will keep the data from each keypad separate."))
  Tbug(string(13, "All the characters received on an individual line will be transmitted"))
  Tbug(string(13, "from the Black Box after an end of message character is"))
  Tbug(string(13, "received on the individual line. "))

  characterIndex := 0
  repeat localIndex from 0 to _Output
    doneFlag[localIndex] := 0
    
  localIndex := @padFakeOutput01  
  repeat while doneFlag[_Output] < _Output
    doneFlag[_Output] := 0 ' use to count keypads that are done          
    repeat outputIndex from 0 to _PadG
      if doneFlag[outputIndex] == 0 ' if zero then this pad hasn't reached end of message character                     
        waitcnt(clkfreq / 2 + cnt) ' keep things slow enough to watch. 
        Tbug(string(13, "Output from pad"))
        TbugTx(prefixList[outputIndex])
        Tbug(string(" = ")) 
        TbugTx(byte[localIndex + (outputIndex * _KeyPressBufferSize)])' Send data to terminal window  
        Tx(outputIndex, byte[localIndex + (outputIndex * _KeyPressBufferSize)]) ' Send data to black box
        if byte[localIndex + (outputIndex * _KeyPressBufferSize)] == _EndOfMessagePad '"#"
          doneFlag[outputIndex] := 1
      else
        doneFlag[_Output]++ ' add up keypads that are done.
    localIndex++  ' increment to next character in each keypad string.   
    waitcnt(clkfreq * 2 + cnt) ' keep things slow enough to watch.
  Tbug(string(13, "End of interleave test."))
  Tbug(string(13, "Hopefull the ", 34, "Black Box", 34, " kept the individual"))
  Tbug(string(13, "keypad data separate."))
  waitcnt(clkfreq * 3 + cnt) 
  Tbug(string(13, "Let's try that again without delays."))
  Tbug(string(13, "Start of fast interleave test."))
  waitcnt(clkfreq * 1 + cnt) 
  Tbug(string(13, "On your mark."))
  waitcnt(clkfreq * 1 + cnt) 
  Tbug(string(13, "Get set."))
  waitcnt(clkfreq * 1 + cnt) 
  Tbug(string(13, "GO", 13))
  
  characterIndex := 0
  repeat localIndex from 0 to _Output
    doneFlag[localIndex] := 0
    
  localIndex := @padFakeOutput01  
  repeat while doneFlag[_Output] < _Output
    doneFlag[_Output] := 0 ' use to count keypads that are done          
    repeat outputIndex from 0 to _PadG
      if doneFlag[outputIndex] == 0 ' if zero then this pad hasn't reached end of message character
        Tbug(string(13, "Output from pad"))
        TbugTx(prefixList[outputIndex])
        Tbug(string(" = ")) 
        TbugTx(byte[localIndex + (outputIndex * _KeyPressBufferSize)])' Send data to terminal window  
        Tx(outputIndex, byte[localIndex + (outputIndex * _KeyPressBufferSize)]) ' Send data to black box
        if byte[localIndex + (outputIndex * _KeyPressBufferSize)] == _EndOfMessagePad '"#"
          doneFlag[outputIndex] := 1
      else
        doneFlag[_Output]++ ' add up keypads that are done.
    localIndex++  ' increment to next character in each keypad string.
  Tbug(string(13, "End of fast interleave test."))
  Tbug(string(13, "Hopefull the ", 34, "Black Box", 34, " kept the individual"))
  Tbug(string(13, "keypad data separate (again)."))
  waitcnt(clkfreq * 3 + cnt) 

  Tbug(string(13, "Use data in ", 34, "sequence", 34, " it indicate which keypad to simulate.")) 
  Tbug(string(13, "Use data in the ", 34, "delay", 34, " array to determine "))  
  Tbug(string(13, "how long to delay between sending data sets.")) 
  Tbug(string(13, "This is the final test.")) 
  Tbug(string(13, "This test will not stop without outside intervention."))
  waitcnt(clkfreq * 2 + cnt)
  localSize := strsize(@sequence)
  repeat
    repeat localIndex from 0 to localSize - 1
      if delayIndex => _NumberOfDelayChoices
        delayIndex := 0
      if outputIndex => _NumberOfOutputChoices
        outputIndex := 0 
      activePad := sequence[localIndex] - "A"
      Str(activePad, @padFakeOutput00 + (outputIndex * _KeyPressBufferSize))
      Tbug(string(13, "Output from pad"))
      TbugTx(prefixList[activePad])
      Tbug(string(" = "))
      Tbug(@padFakeOutput00 + (outputIndex * _KeyPressBufferSize))
      outputIndex++
      waitcnt(delay[delayIndex++] + cnt)

PRI Str(localComIndex, localStringPtr)

  if localComIndex < _SerialPortGroupSize
    Com0.str(localComIndex, localStringPtr)
  elseif localComIndex < _SerialPortGroupSize * 2
    Com1.str(localComIndex - _SerialPortGroupSize, localStringPtr) 
  else
    Tbug(string(13, _BlackBoxMessagePrefix, " Str error, Com Index out of range.", _EndOfMessagePad))

PRI Tx(localComIndex, localCharacter)

  if localComIndex < _SerialPortGroupSize
    Com0.tx(localComIndex, localCharacter)
  elseif localComIndex < _SerialPortGroupSize * 2
    Com1.tx(localComIndex - _SerialPortGroupSize, localCharacter) 
  else
    Tbug(string(13, _BlackBoxMessagePrefix, " Tx error, Com Index out of range.", _EndOfMessagePad))
  
PRI Tbug(localPtr)

  'Tv.str(localPtr)
  Com1.str(_Output - _SerialPortGroupSize, localPtr)
      
PRI TbugDec(localDec)

  'Tv.dec(localDec)
  Com1.dec(_Output - _SerialPortGroupSize, localDec)
  
PRI TbugTx(localCharacter)
'' Tbug... methods make it easier to add multiple debug
'' monitors (such as a TV).

  if (localCharacter > 31 and localCharacter < 127) or localCharacter == 13
    'Tv.out(localCharacter)
    Com1.tx(_Output - _SerialPortGroupSize, localCharacter)    
  else
    'Tv.out(60)
    'Tv.out(36)
    Com1.tx(_Output - _SerialPortGroupSize, 60)
    Com1.tx(_Output - _SerialPortGroupSize, 36)
    TbugHex(localCharacter, 2)
    'Tv.out(62)
    Com1.tx(_Output - _SerialPortGroupSize, 62)
 
PRI TbugHex(localHex, localSize)
 
  'Tv.hex(localHex, localSize)
  Com1.hex(_Output - _SerialPortGroupSize, localHex, localSize)

DAT
                             '0123456
prefixList              byte "ABCDEFG", _BlackBoxMessagePrefix
'' prefixList lists the prefixs to add to outgoing messages.
'' The prefix used is determined by which serial line the
'' data is received on.

sequence                byte "CBAGFEDBBAAEEDDFFGGCC", 0
'' sequence is a list of keypads to simulate.  The program
'' uses this list with the data below to generate simulated
'' input from keypads.



'' Each padFakeOutputXX array should have exactly _KeyPressBufferSize
'' members.  If the keypad data is less than _KeyPressBufferSize
'' characters long, then add zeros to pad the array so it is exactly
'' _KeyPressBufferSize characters long.
                             '          1111111111
                             '01234567890123456789
padFakeOutput00         byte "123*4567#", 0[11] 
padFakeOutput01         byte "1111#", 0[15] 
padFakeOutput02         byte "2222#", 0[15] 
padFakeOutput03         byte "3333#", 0[15] 
padFakeOutput04         byte "4444#", 0[15] 
padFakeOutput05         byte "9999#", 0[15] 
padFakeOutput06         byte "0000#", 0[15] 
padFakeOutput07         byte "1234567890#", 0[9] 
padFakeOutput08         byte "4321#", 0[15] 
                             '          1111111111
                             '01234567890123456789
padFakeOutput09         byte "987*654*3210#", 0[7]
                                            